import {
	GetServerSideProps,
	GetServerSidePropsContext,
	NextApiHandler,
	PreviewData,
} from 'next';
import { ParsedUrlQuery } from 'querystring';
import { NextApiRequest as DefaultNextApiRequest } from 'next/dist/shared/lib/utils';
import { EncTypeDeserializerMap } from '../../utils/serialization';
import { IceformNextServerResponse } from '../response';

export type OnActionFunction<
	Props extends Record<string, unknown> = Record<string, unknown>,
	Params extends ParsedUrlQuery = ParsedUrlQuery,
	Preview extends PreviewData = PreviewData,
> = (
	actionReq: DefaultNextApiRequest,
	actionRes: IceformNextServerResponse,
	context: GetServerSidePropsContext<Params, Preview>,
) => Promise<Awaited<ReturnType<GetServerSideProps<Props, Params, Preview>>> | undefined>;

export interface ActionWrapperOptions {
	fn: NextApiHandler,
	onAction?: OnActionFunction,
	deserializers?: EncTypeDeserializerMap,
	methodFormKey?: string,
	requestIdFormKey?: string,
	preventRedirectFormKey?: string,
	/**
	 * Maps the Location header from the handler response to an accessible URL.
	 * @param url
	 */
	mapLocationToRedirectDestination?: (referer: string, url: string) => string,
}
