interface AbstractFormDataItem<T> {
	value: T | null;
}

export interface FileItem extends AbstractFormDataItem<Buffer> {
	kind: 'file';
	mimeType: string;
	filename: string;
}

export interface FieldItem extends AbstractFormDataItem<string> {
	kind: 'field';
}

export type FormDataItem = FileItem | FieldItem;

export type FormDataMap = Record<string, FormDataItem>;

export const addFormObjectBodyHelpers = (deserialized: Record<string, unknown>): FormDataMap => (
	Object.fromEntries(
		Object
			.entries(deserialized)
			.filter(([, value]) => typeof value !== 'undefined')
			.map(([key, value]) => [
				key,
				{
					kind: 'field',
					value: value?.toString() ?? null,
				},
			] satisfies [string, FieldItem]),
	)
);

export const removeFormObjectBodyHelpers = (formObject: FormDataMap): Record<string, unknown> => (
	Object.fromEntries(
		Object
			.entries(formObject)
			.filter(([, value]) => typeof value !== 'undefined')
			.map(([key, valueWithHelper]) => [
				key,
				valueWithHelper.value,
			]),
	)
);
