import { IncomingMessage } from 'http';
import busboy from 'busboy';

export const getBody = (req: IncomingMessage) => new Promise<Buffer>((resolve, reject) => {
	let body = Buffer.from('');

	req.on('data', (chunk) => {
		body = Buffer.concat([body, chunk]);
	});

	req.on('error', (err) => {
		reject(err);
	});

	req.on('end', () => {
		resolve(body);
	});
});

export const parseMultipartFormData = async (
	req: IncomingMessage,
) => new Promise<Record<string, unknown>>((resolve, reject) => {
	const body: Record<string, unknown> = {};
	const bb = busboy({
		headers: req.headers,
	});

	bb.on('file', (name, file, info) => {
		const {
			filename,
			mimeType,
		} = info;
		let buffer = Buffer.from('');

		file.on('data', (data) => {
			buffer = Buffer.concat([buffer, data]);
		});

		file.on('close', () => {
			// filename can be undefined somehow...
			if (!filename && buffer.length <= 0) {
				return;
			}

			const bufferMut = buffer as unknown as Record<string, unknown>;
			bufferMut.name = filename;
			bufferMut.type = mimeType;
			bufferMut.size = buffer.length;

			body[name] = bufferMut;
		});
	});

	bb.on('field', (name, value) => {
		// TODO max length for long values, convert to reference instead
		body[name] = value;
	});

	bb.on('close', () => {
		resolve(body);
	});

	bb.on('error', (error) => {
		reject(error);
	});

	req.pipe(bb);
});
